package main

import (
	"crypto/sha1"
	"os"
	"io"
	"bytes"
	"encoding/base64"
)

const NCPU = 4

type Checksum struct {
	Name string
	Hash string
}

type ChecksumIterator Path

// A Iterable kielegitese vegett es a checksum letrehozas vegett
func (path ChecksumIterator) Iter() <-chan Checksum {	
	res := make(chan Checksum)
	
	// ez a nagyobb fuggveny kezeli a dolgokat, egy masik goroutinban
	go func() {
		// megcsinaljuk a szemaforunkat, a CPU darabszamokra amiket tamogatunk
		sem := make(chan int, NCPU)
		
		// benepesitjuk a szemaforunkat a thread IDkal (cpu 1, cpu 2, stb.)
		for i := 0; i < NCPU; i += 1 { sem <- i }
		
		// most vegigiteralunk a fajlokon amiket talalunk, es mindegyikhez
		// egy processt rendelunk, olyan CPU IDvel, ami eppen keznel van
		for file := range Path(path).Iter() {
			go process(file, res, sem, <- sem)
		}
		
		// amikor ideerunk, mar kifogytunk a fajlokbol amiket checksumolni kell
		// ezert bezarjuk az iteraciot
		close(res)
	}()
	
	return res
}

func process(path Path, out chan<- Checksum, res chan<- int, threadId int) {
	// biztositsuk, hogy jelzunk a szemafornak, amikor keszen vagyunk
	defer func() { res <- threadId }()
	
	// kuldjuk ki a Checksumot a csatornabol, ha nem volt error
	if data, err := getHash(path); err == nil {	
		out <- Checksum{path.String(), data}	
	}
}

// Egy path stringbol es egy Hash interfacebol, kiszamitsuk
// a fajl tartalmanak hashet
func getHash(path Path) (result string, err os.Error) {
	// uj hash szamito
	hash := sha1.New()
	
	// ha ki tudjuk nyitni a fajlt...
	if file, err := os.Open(string(path), os.O_RDONLY, 0); err == nil {
		// es be tudjuk masolni a tartalmat a hashelesre
		if _, err := io.Copy(hash, file); err == nil {
			// akkor van egy eredmenyunk
			result = encodeBase64(hash.Sum())
		}
	}
	// return, amennyiben nil az eredmeny, vagy error
	return
}

// kiszedjuk a stringet, egy byte tombbol
func encodeBase64(source []byte) string {
	// keszitunk egy byte szeletet, pont akkorat, hogy a kodolasi eredmeny beleferjen 
	dest := make([]byte, base64.StdEncoding.EncodedLen(len(source)))
	// kodoljuk
	base64.StdEncoding.Encode(dest, source)
	// atalkitsuk ezt a byte buffert stringre
	return bytes.NewBuffer(dest).String()
}
